#include <cstdio>
#include <cstdlib>


class Circle {
    float rad;
    int x,y;
public:
    // default constructor
    //   Circle c;
    //   Circle c = Circle();
    //   Circle* c = new Circle();
    // 
    Circle(){
        rad = 0;
        x = y = 0;
        printf("Circle()\n");
    }

    // constructor with parameters.
    //   Circle c(10,5,7);
    //   Circle c = Circle(10,5,7);
    //   Circle* c = new Circle(10,5,7);
    // 
    Circle(float r,int a,int b){
        rad = r;
        x = a; 
        y = b;
        printf("Circle(%f,%d,%d)\n",rad,x,y);
    }

    // copy constructor (if you don't declare it, C++ will
    // create a "default" one that will just make a copy of
    // the data.
    //   Circle a = b;
    //   Circle a(b);
    //   Circle a = Circle(b);
    //   Circle *a = new Circle(b);
    Circle(const Circle& a){  // a is a reference to passed object.
        rad = a.rad;
        x = a.x;
        y = a.y;
        printf("Circle( circle object )\n");
    }
    
    // assignment operator 
    //   Circle a,b;
    //   a = b;     // once objects exist, assignment works
    //      Note that this operator returns a reference
    //      (we don't want to create another object just
    //       to return it).
    Circle& operator=(const Circle& a){
        rad = a.rad;
        x = a.x;
        y = a.y;
        printf("operator=( circle object )\n");
        return *this;       // to enable chained assignment; a=b=c;
    }


    // destructor
    // gets called when object goes out of scope.
    // gets called when delete is called.
    //
    ~Circle(){
        rad = 0;
        x = y = 0;
        printf("~Circle()\n");
    }

};


int main(){
    
    {
        printf("declare: Circle c;\n");
        Circle c;
    }
    
    {
        printf("declare: Circle c = Circle();\n");
        Circle c = Circle();
    }

    {
        printf("declare: Circle* c;\n");    
        Circle* c;
    }


    {
        printf("declare: Circle* c = new Circle();\n");    
        Circle* c = new Circle();

        printf("calling delete c;\n");    
        delete c;

        printf("at end of scope...\n");
    }


    {
        printf("declare: Circle c(10,5,7);\n");
        Circle c(10,5,7);
    }
    
    {
        printf("declare: Circle c = Circle(10,5,7);\n");
        Circle c = Circle(10,5,7);
    }

    {
        printf("declare: Circle* c = new Circle(10,5,7);\n");    
        Circle* c = new Circle(10,5,7);

        printf("calling delete c;\n");    
        delete c;

        printf("at end of scope...\n");
    }

    {
        printf("declare: Circle c(2,1,3);\n");
        Circle c(2,1,3);

        printf("declare: Circle d = c;\n"); 
        Circle d = c;
    }

    {
        printf("declare: Circle c(2,1,3);\n");
        Circle c(2,1,3);

        printf("declare: Circle d;\n"); 
        Circle d;

        printf("declare: d = c;    (assignment operator)\n"); 
        d = c;      // assignment 

        printf("declare: Circle e;\n"); 
        Circle e;

        printf("declare: e = d = c;  (assignment operator)\n"); 
        e = d = c;      // assignment 
    }


    return 0;
}



